"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const request = require('request-promise-native');
const lodash_1 = require("lodash");
const pkg = require("pjson");
const timeout = 4000;
const headers = [
    {
        name: 'User-Agent',
        value: `Pastebin-ts/${pkg.version}`,
    },
    {
        name: 'Cache-Control',
        value: 'no-cache',
    },
];
const getOptions = (method, params = {}) => {
    const options = {
        resolveWithFullResponse: true,
        method,
        headers,
        timeout,
        followRedirect: true,
    };
    if (method === 'GET') {
        options.qs = params;
    }
    else if (method === 'POST') {
        options.form = params;
    }
    return options;
};
exports.getRequest = (path = '', params) => new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
    const options = getOptions('GET', params);
    if (path === '') {
        return reject(new Error('No path provided!'));
    }
    let response;
    try {
        response = yield request(path, options);
    }
    catch (e) {
        if (e.statusCode && e.statusCode === 404) {
            return reject(new Error('Error 404, not found'));
        }
        return reject(new Error(e));
    }
    const { statusCode, body } = response;
    if (lodash_1.isNull(statusCode) || lodash_1.isUndefined(statusCode)) {
        return reject(new Error('Unknown status'));
    }
    else if (statusCode === 404) {
        return reject(new Error('Error 404, not found'));
    }
    else if (statusCode !== 200) {
        return reject(new Error(`Unknown error, status: ${statusCode}`));
    }
    else if (lodash_1.isNull(body) || lodash_1.isUndefined(body) || body === '') {
        return reject(new Error('Empty response'));
    }
    if (body.indexOf('Bad API request') !== -1) {
        return reject(new Error(body));
    }
    resolve(body);
}));
exports.postRequest = (path = '', params) => new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
    const options = getOptions('POST', params);
    if (path === '') {
        return reject(new Error('No path provided!'));
    }
    let response;
    try {
        response = yield request(path, options);
    }
    catch (e) {
        if (e.statusCode && e.statusCode === 404) {
            return reject(new Error('Error 404, not found'));
        }
        return reject(new Error(e));
    }
    const { statusCode, body } = response;
    if (lodash_1.isNull(statusCode) || lodash_1.isUndefined(statusCode)) {
        return reject(new Error('Unknown status'));
    }
    else if (statusCode === 404) {
        return reject(new Error('Error 404, not found'));
    }
    else if (statusCode !== 200) {
        return reject(new Error(`Unknown error, status: ${statusCode}`));
    }
    else if (lodash_1.isNull(body) || lodash_1.isUndefined(body) || body === '') {
        return reject(new Error('Empty response'));
    }
    else if (body.indexOf('Bad API request') !== -1) {
        return reject(new Error(`Error: ${body}`));
    }
    else if (body.indexOf('Post limit') !== -1) {
        return reject(new Error(`Error: ${body}`));
    }
    resolve(body);
}));
